import random

import pygame

import driver
from vector import Vector
from gameover import GameOverState

# Actual G
#G = 6.67384e-11

# G assuming everything is 10^11 times more massive
G = 6.67

TRAIL_TIME = 0.1

class Body(object):
    
    def __init__(self):
        self.emits_gravity = False
        self.receives_gravity = False
        self.radius = 10
        self.mass = 100
        self.color = (255, 255, 255)
        self.loc = Vector()
        self.velocity = Vector()
        self.trail = []
        self.time_to_trail = 0
    
    def draw(self, screen, scale=1.0):
        x = int(self.loc.x)
        y = int(self.loc.y)
        pygame.draw.circle(screen, self.color, (x,y),
                           self.radius)
        
        for entry in self.trail:
            screen.fill(self.color, entry)

    def colliding_with(self, other):
        sum = self.radius + other.radius
        if self.loc.dist2(other.loc) < sum*sum:
            return True
        return False

    def update(self, delay, others):
        self.loc += self.velocity * delay
        
        self.time_to_trail -= delay
        if self.time_to_trail <= 0:
            self.time_to_trail += TRAIL_TIME
            x,y = self.loc.as_int_tuple()
            r = pygame.Rect(x, y, 1, 1)
            self.trail.append(r)
        
        # Modify velocity
        if self.receives_gravity:
            impulse = Vector()
            for other in others:
                if other == self:
                    continue
                f = G * self.mass * other.mass / self.loc.dist2(other.loc)
                scaled_f = f * delay
                impulse += (other.loc - self.loc) * scaled_f
            self.velocity += impulse
            

class PlayingGameState(driver.State):
    
    def __init__(self, state_driver, difficult = False):
        super(PlayingGameState, self).__init__(state_driver)
        
        self.drawables = []
        self.difficult = difficult

        # Create a central planet
        kerbin = Body()
        kerbin.emits_gravity = True
        kerbin.radius = 15
        kerbin.loc.x = state_driver.w / 2
        kerbin.loc.y = state_driver.h / 2
        
        self.kerbin = kerbin
        self.add(kerbin)
        
        self.activemoon = None
        
        self.score = 0
        self.font = pygame.font.Font(None, 24)
        self.update_moons_released()
        self.update_score()
    
    def add(self, body):
        self.drawables.append(body)

    def draw(self, screen):
        for body in self.drawables:
            body.draw(screen)
        
        if self.activemoon:
            # Draw a line between the moon's location and the mouse
            pygame.draw.line(screen, (255, 255, 255),
                             self.activemoon.loc.as_int_tuple(),
                             pygame.mouse.get_pos());
        
        # Text info
        screen.blit(self.surface_moons_released, (0, 0))
        screen.blit(self.surface_score,
                    (self.driver.w - self.surface_score.get_width(), 0))

    def mouse(self, pos, button, pressed):
        if pressed and button < 4:
            moon = Body()
            moon.mass = 10
            moon.loc = Vector(tuple=pos)
            r = random.randint(64, 255)
            g = random.randint(64, 255)
            b = random.randint(64, 255)
            moon.color = (r, g, b)
            self.add(moon)
            self.activemoon = moon
        elif self.activemoon:
            self.activemoon.receives_gravity = True
            self.activemoon.emits_gravity = self.difficult
            mouse = Vector(tuple=pygame.mouse.get_pos())
            velocity = mouse - self.activemoon.loc
            
            self.activemoon.velocity = velocity
            self.activemoon = None
            self.update_moons_released()

    def update(self, delay):
        emitters = [d for d in self.drawables if d.emits_gravity]
        gameover = False
        for body in self.drawables:
            body.update(delay, emitters)
            for other in self.drawables:
                if other is not body and other.colliding_with(body):
                    gameover = True
            
            # Make sure it's still on the screen
            if (body.loc.x < 0 or body.loc.x > self.driver.w or
                body.loc.y < 0 or body.loc.y > self.driver.h):
                gameover = True
        
        if gameover:
            self.driver.replace(GameOverState(self.driver, self))
        
        # Score!
        moons = len(self.drawables) - 1
        self.score += delay * moons * 10
        self.update_score()

    def update_moons_released(self):
        text = "Moons released: %d" % (len(self.drawables) - 1)
        self.surface_moons_released = self.font.render(text, False,
                                                       (255, 255, 255))
    
    def update_score(self):
        text = "Score: %d" % int(self.score)
        self.surface_score = self.font.render(text, False, (255, 255, 255))
